@@ -37,10 +37,6 @@
 #include <net/if.h>
 #endif
 
-#ifdef ANDROID
-#define statvfs statfs
-#endif
-
 #ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
@@ -49,6 +45,10 @@
 #include <sys/statvfs.h>
 #endif
 
+#if defined(__ANDROID__) && !defined(HAVE_SYS_STATVFS_H)
+#define statvfs statfs
+#endif
+
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
@@ -110,6 +110,7 @@ struct sync_cb_data {
 static void wait_for_reply(struct rpc_context *rpc, struct sync_cb_data *cb_data)
 {
 	struct pollfd pfd;
+	int ret;
 
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
@@ -117,16 +118,24 @@ static void wait_for_reply(struct rpc_context *rpc, struct sync_cb_data *cb_data
 
 		pfd.fd = rpc_get_fd(rpc);
 		pfd.events = rpc_which_events(rpc);
-		if (poll(&pfd, 1, -1) < 0) {
+
+		ret =  poll(&pfd, 1, rpc_get_timeout(rpc));
+		if (ret < 0) {
 			rpc_set_error(rpc, "Poll failed");
 			cb_data->status = -EIO;
 			break;
+		} else if(ret == 0) {
+			rpc_set_error(rpc, "Timed out after [%d] milliseconds",rpc_get_timeout(rpc));
+			cb_data->status = -EIO;
+			break;
 		}
+
 		if (rpc_service(rpc, pfd.revents) < 0) {
 			rpc_set_error(rpc, "rpc_service failed");
 			cb_data->status = -EIO;
 			break;
 		}
+
 		if (rpc_get_fd(rpc) == -1) {
 			rpc_set_error(rpc, "Socket closed\n");
 			break;
@@ -137,16 +146,24 @@ static void wait_for_reply(struct rpc_context *rpc, struct sync_cb_data *cb_data
 static void wait_for_nfs_reply(struct nfs_context *nfs, struct sync_cb_data *cb_data)
 {
 	struct pollfd pfd;
+	int ret = -1;
 
 	while (!cb_data->is_finished) {
 
 		pfd.fd = nfs_get_fd(nfs);
 		pfd.events = nfs_which_events(nfs);
-		if (poll(&pfd, 1, -1) < 0) {
+
+		ret =  poll(&pfd, 1, nfs_get_timeout(nfs));
+		if (ret < 0) {
 			nfs_set_error(nfs, "Poll failed");
 			cb_data->status = -EIO;
 			break;
+		} else if(ret == 0) {
+			nfs_set_error(nfs, "Timed out after [%d] milliseconds",nfs_get_timeout(nfs));
+			cb_data->status = -EIO;
+			break;
 		}
+
 		if (nfs_service(nfs, pfd.revents) < 0) {
 			nfs_set_error(nfs, "nfs_service failed");
 			cb_data->status = -EIO;
@@ -157,9 +174,6 @@ static void wait_for_nfs_reply(struct nfs_context *nfs, struct sync_cb_data *cb_
 
 
 
-
-
-
 /*
  * connect to the server and mount the export
  */
@@ -457,7 +471,11 @@ int nfs_close(struct nfs_context *nfs, struct nfsfh *nfsfh)
 /*
  * fstat()
  */
+#ifdef WIN32
+int nfs_fstat(struct nfs_context *nfs, struct nfsfh *nfsfh, struct __stat64 *st)
+#else
 int nfs_fstat(struct nfs_context *nfs, struct nfsfh *nfsfh, struct stat *st)
+#endif
 {
 	struct sync_cb_data cb_data;
 
@@ -1689,39 +1707,39 @@ static int send_nfsd_probes(struct rpc_context *rpc, struct ifconf *ifc, struct
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
 	for (ptr =(char *)(ifc->ifc_buf); ptr < (char *)(ifc->ifc_buf) + ifc->ifc_len; ) {
-		struct ifreq *ifr;
+		struct ifreq ifr;
 		char bcdd[16];
 
-		ifr = (struct ifreq *)ptr;
+		memcpy(&ifr, ptr, sizeof(struct ifreq));
 #ifdef HAVE_SOCKADDR_LEN
-		if (ifr->ifr_addr.sa_len > sizeof(struct sockaddr)) {
-			ptr += sizeof(ifr->ifr_name) + ifr->ifr_addr.sa_len;
+		if (ifr.ifr_addr.sa_len > sizeof(struct sockaddr)) {
+			ptr += sizeof(ifr.ifr_name) + ifr.ifr_addr.sa_len;
 		} else {
-			ptr += sizeof(ifr->ifr_name) + sizeof(struct sockaddr);
+			ptr += sizeof(ifr.ifr_name) + sizeof(struct sockaddr);
 		}
 #else
 		ptr += sizeof(struct ifreq);
 #endif
 
-		if (ifr->ifr_addr.sa_family != AF_INET) {
+		if (ifr.ifr_addr.sa_family != AF_INET) {
 			continue;
 		}
-		if (ioctl(rpc_get_fd(rpc), SIOCGIFFLAGS, ifr) < 0) {
+		if (ioctl(rpc_get_fd(rpc), SIOCGIFFLAGS, &ifr) < 0) {
 			return -1;
 		}
-		if (!(ifr->ifr_flags & IFF_UP)) {
+		if (!(ifr.ifr_flags & IFF_UP)) {
 			continue;
 		}
-		if (ifr->ifr_flags & IFF_LOOPBACK) {
+		if (ifr.ifr_flags & IFF_LOOPBACK) {
 			continue;
 		}
-		if (!(ifr->ifr_flags & IFF_BROADCAST)) {
+		if (!(ifr.ifr_flags & IFF_BROADCAST)) {
 			continue;
 		}
-		if (ioctl(rpc_get_fd(rpc), SIOCGIFBRDADDR, ifr) < 0) {
+		if (ioctl(rpc_get_fd(rpc), SIOCGIFBRDADDR, &ifr) < 0) {
 			continue;
 		}
-		if (getnameinfo(&ifr->ifr_broadaddr, sizeof(struct sockaddr_in), &bcdd[0], sizeof(bcdd), NULL, 0, NI_NUMERICHOST) < 0) {
+		if (getnameinfo(&ifr.ifr_broadaddr, sizeof(struct sockaddr_in), &bcdd[0], sizeof(bcdd), NULL, 0, NI_NUMERICHOST) < 0) {
 			continue;
 		}
 		if (rpc_set_udp_destination(rpc, bcdd, 111, 1) < 0) {
