@@ -84,26 +84,41 @@ unsigned int rpc_hash_xid(uint32_t xid)
 	return (xid * 7919) % HASHES;
 }
 
-struct rpc_pdu *rpc_allocate_pdu(struct rpc_context *rpc, int program, int version, int procedure, rpc_cb cb, void *private_data, zdrproc_t zdr_decode_fn, int zdr_decode_bufsize)
+#define PAD_TO_8_BYTES(x) ((x + 0x07) & ~0x07)
+
+struct rpc_pdu *rpc_allocate_pdu2(struct rpc_context *rpc, int program, int version, int procedure, rpc_cb cb, void *private_data, zdrproc_t zdr_decode_fn, int zdr_decode_bufsize, size_t alloc_hint)
 {
 	struct rpc_pdu *pdu;
 	struct rpc_msg msg;
+	int pdu_size;
 
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
-	pdu = malloc(sizeof(struct rpc_pdu));
+	/* Since we already know how much buffer we need for the decoding
+	 * we can just piggyback in the same alloc as for the pdu.
+	 */
+	pdu_size = PAD_TO_8_BYTES(sizeof(struct rpc_pdu));
+	pdu_size += PAD_TO_8_BYTES(zdr_decode_bufsize);
+
+	pdu = malloc(pdu_size);
 	if (pdu == NULL) {
 		rpc_set_error(rpc, "Out of memory: Failed to allocate pdu structure");
 		return NULL;
 	}
-	memset(pdu, 0, sizeof(struct rpc_pdu));
+	memset(pdu, 0, pdu_size);
 	pdu->xid                = rpc->xid++;
 	pdu->cb                 = cb;
 	pdu->private_data       = private_data;
 	pdu->zdr_decode_fn      = zdr_decode_fn;
 	pdu->zdr_decode_bufsize = zdr_decode_bufsize;
 
-	zdrmem_create(&pdu->zdr, rpc->encodebuf, rpc->encodebuflen, ZDR_ENCODE);
+	pdu->outdata.data = malloc(ZDR_ENCODEBUF_MINSIZE + alloc_hint);
+	if (pdu->outdata.data == NULL) {
+		rpc_set_error(rpc, "Out of memory: Failed to allocate encode buffer");
+		return NULL;
+	}
+
+	zdrmem_create(&pdu->zdr, pdu->outdata.data, ZDR_ENCODEBUF_MINSIZE + alloc_hint, ZDR_ENCODE);
 	if (rpc->is_udp == 0) {
 		zdr_setpos(&pdu->zdr, 4); /* skip past the record marker */
 	}
@@ -129,6 +144,11 @@ struct rpc_pdu *rpc_allocate_pdu(struct rpc_context *rpc, int program, int versi
 	return pdu;
 }
 
+struct rpc_pdu *rpc_allocate_pdu(struct rpc_context *rpc, int program, int version, int procedure, rpc_cb cb, void *private_data, zdrproc_t zdr_decode_fn, int zdr_decode_bufsize)
+{
+	return rpc_allocate_pdu2(rpc, program, version, procedure, cb, private_data, zdr_decode_fn, zdr_decode_bufsize, 0);
+}
+
 void rpc_free_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu)
 {
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
@@ -140,7 +160,6 @@ void rpc_free_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu)
 
 	if (pdu->zdr_decode_buf != NULL) {
 		zdr_free(pdu->zdr_decode_fn, pdu->zdr_decode_buf);
-		free(pdu->zdr_decode_buf);
 		pdu->zdr_decode_buf = NULL;
 	}
 
@@ -167,7 +186,7 @@ int rpc_queue_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu)
 		unsigned int hash;
 
 // XXX add a rpc->udp_dest_sock_size  and get rid of sys/socket.h and netinet/in.h
-		if (sendto(rpc->fd, rpc->encodebuf, size, MSG_DONTWAIT, rpc->udp_dest, sizeof(struct sockaddr_in)) < 0) {
+		if (sendto(rpc->fd, pdu->zdr.buf, size, MSG_DONTWAIT, rpc->udp_dest, sizeof(struct sockaddr_in)) < 0) {
 			rpc_set_error(rpc, "Sendto failed with errno %s", strerror(errno));
 			rpc_free_pdu(rpc, pdu);
 			return -1;
@@ -184,24 +203,16 @@ int rpc_queue_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu)
 	zdr_int(&pdu->zdr, &recordmarker);
 
 	pdu->outdata.size = size;
-	pdu->outdata.data = malloc(pdu->outdata.size);
-	if (pdu->outdata.data == NULL) {
-		rpc_set_error(rpc, "Out of memory. Failed to allocate buffer for pdu\n");
-		rpc_free_pdu(rpc, pdu);
-		return -1;
-	}
-
-	memcpy(pdu->outdata.data, rpc->encodebuf, pdu->outdata.size);
 	rpc_enqueue(&rpc->outqueue, pdu);
 
 	return 0;
 }
 
-int rpc_get_pdu_size(char *buf)
+uint32_t rpc_get_pdu_size(char *buf)
 {
 	uint32_t size;
 
-	size = ntohl(*(uint32_t *)buf);
+	size = ntohl(*(uint32_t *)(void *)buf);
 
 	return (size & 0x7fffffff) + 4;
 }
@@ -215,19 +226,7 @@ static int rpc_process_reply(struct rpc_context *rpc, struct rpc_pdu *pdu, ZDR *
 	memset(&msg, 0, sizeof(struct rpc_msg));
 	msg.body.rbody.reply.areply.verf = _null_auth;
 	if (pdu->zdr_decode_bufsize > 0) {
-		if (pdu->zdr_decode_buf != NULL) {
-			free(pdu->zdr_decode_buf);
-		}
-		pdu->zdr_decode_buf = malloc(pdu->zdr_decode_bufsize);
-		if (pdu->zdr_decode_buf == NULL) {
-			rpc_set_error(rpc, "Failed to allocate memory for "
-				      "zdr_encode_buf in rpc_process_reply");
-			pdu->cb(rpc, RPC_STATUS_ERROR, "Failed to allocate "
-				"buffer for decoding of ZDR reply",
-				pdu->private_data);
-			return 0;
-		}
-		memset(pdu->zdr_decode_buf, 0, pdu->zdr_decode_bufsize);
+		pdu->zdr_decode_buf = (char *)pdu + PAD_TO_8_BYTES(sizeof(struct rpc_pdu));
 	}
 	msg.body.rbody.reply.areply.reply_data.results.where = pdu->zdr_decode_buf;
 	msg.body.rbody.reply.areply.reply_data.results.proc  = pdu->zdr_decode_fn;
@@ -238,7 +237,6 @@ static int rpc_process_reply(struct rpc_context *rpc, struct rpc_pdu *pdu, ZDR *
 		pdu->cb(rpc, RPC_STATUS_ERROR, "Message rejected by server",
 			pdu->private_data);
 		if (pdu->zdr_decode_buf != NULL) {
-			free(pdu->zdr_decode_buf);
 			pdu->zdr_decode_buf = NULL;
 		}
 		return 0;
