@@ -105,7 +105,20 @@ static void set_nolinger(int fd)
 	struct linger lng;
 	lng.l_onoff = 1;
 	lng.l_linger = 0;
-	setsockopt(fd, SOL_SOCKET, SO_LINGER, &lng, sizeof(lng));
+	setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *)&lng, sizeof(lng));
+}
+
+static int set_bind_device(int fd, char *ifname)
+{
+	int rc = 0;
+
+#ifdef HAVE_SO_BINDTODEVICE
+	if (*ifname) {
+		rc = setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE, ifname, strlen(ifname));
+	}
+
+#endif
+	return rc;
 }
 
 #ifdef HAVE_NETINET_TCP_H
@@ -180,7 +193,7 @@ static int rpc_write_to_socket(struct rpc_context *rpc)
 
 		total = pdu->outdata.size;
 
-		count = send(rpc->fd, pdu->outdata.data + pdu->written, total - pdu->written, 0);
+		count = send(rpc->fd, pdu->outdata.data + pdu->written, (int)(total - pdu->written), 0);
 		if (count == -1) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK) {
 				return 0;
@@ -207,9 +220,8 @@ static int rpc_write_to_socket(struct rpc_context *rpc)
 #define MAX_UDP_SIZE 65536
 static int rpc_read_from_socket(struct rpc_context *rpc)
 {
-	int size;
-	int pdu_size;
-	int32_t count;
+	uint32_t pdu_size;
+	ssize_t count;
 
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
@@ -240,35 +252,9 @@ static int rpc_read_from_socket(struct rpc_context *rpc)
 		return 0;
 	}
 
-	/* read record marker, 4 bytes at the beginning of every pdu */
-	if (rpc->inbuf == NULL) {
-		rpc->insize = 4;
-		rpc->inbuf = malloc(rpc->insize);
-		if (rpc->inbuf == NULL) {
-			rpc_set_error(rpc, "Failed to allocate buffer for record marker, errno:%d. Closing socket.", errno);
-			return -1;
-		}
-	}
-
-	if (rpc->inpos < 4) {
-		size = 4 - rpc->inpos;
-
-		count = recv(rpc->fd, rpc->inbuf + rpc->inpos, size, MSG_DONTWAIT);
-		if (count == -1) {
-			if (errno == EINTR || errno == EAGAIN) {
-				return 0;
-			}
-			rpc_set_error(rpc, "Read from socket failed, errno:%d. Closing socket.", errno);
-			return -1;
-		}
-		rpc->inpos += count;
-
-		if (rpc->inpos < 4) {
-			return 0;
-		}
-	}
-
-	pdu_size = rpc_get_pdu_size(rpc->inbuf);
+again:
+	/* read record marker, 4 bytes at the beginning of every pdu first */
+	pdu_size = rpc->inpos < 4 ? 4 : rpc_get_pdu_size(rpc->inbuf);
 
 	if (pdu_size > NFS_MAX_XFER_SIZE + 4096) {
 		rpc_set_error(rpc, "Incoming PDU exceeds limit of %d bytes.", NFS_MAX_XFER_SIZE + 4096);
@@ -284,23 +270,25 @@ static int rpc_read_from_socket(struct rpc_context *rpc)
 		rpc->insize = pdu_size;
 	}
 
-	size = rpc->insize - rpc->inpos;
-	count = recv(rpc->fd, rpc->inbuf + rpc->inpos, size, MSG_DONTWAIT);
-	if (count == -1) {
+	count = recv(rpc->fd, rpc->inbuf + rpc->inpos, rpc->insize - rpc->inpos, MSG_DONTWAIT);
+	if (count < 0) {
 		if (errno == EINTR || errno == EAGAIN) {
 			return 0;
 		}
 		rpc_set_error(rpc, "Read from socket failed, errno:%d. Closing socket.", errno);
 		return -1;
 	}
-
 	if (count == 0) {
 		/* remote side has closed the socket. Reconnect. */
 		return -1;
 	}
-
 	rpc->inpos += count;
 
+	if (rpc->inpos == 4) {
+		/* we have just read the header there is likely more data available */
+		goto again;
+	}
+
 	if (rpc->inpos == rpc->insize) {
 		char *buf = rpc->inbuf;
 
@@ -440,6 +428,11 @@ static int rpc_connect_sockaddr_async(struct rpc_context *rpc, struct sockaddr_s
 	case AF_INET:
 		socksize = sizeof(struct sockaddr_in);
 		rpc->fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+		if (set_bind_device(rpc->fd, rpc->ifname) != 0) {
+			rpc_set_error (rpc, "Failed to bind to interface");
+			return -1;
+		}
+
 #ifdef HAVE_NETINET_TCP_H
 		if (rpc->tcp_syncnt != RPC_PARAM_UNDEFINED) {
 			set_tcp_sockopt(rpc->fd, TCP_SYNCNT, rpc->tcp_syncnt);
@@ -449,6 +442,11 @@ static int rpc_connect_sockaddr_async(struct rpc_context *rpc, struct sockaddr_s
 	case AF_INET6:
 		socksize = sizeof(struct sockaddr_in6);
 		rpc->fd = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP);
+		if (set_bind_device(rpc->fd, rpc->ifname) != 0) {
+			rpc_set_error (rpc, "Failed to bind to interface");
+			return -1;
+		}
+
 #ifdef HAVE_NETINET_TCP_H
 		if (rpc->tcp_syncnt != RPC_PARAM_UNDEFINED) {
 			set_tcp_sockopt(rpc->fd, TCP_SYNCNT, rpc->tcp_syncnt);
@@ -577,7 +575,7 @@ int rpc_connect_async(struct rpc_context *rpc, const char *server, int port, rpc
 	case AF_INET:
 		((struct sockaddr_in *)&rpc->s)->sin_family = ai->ai_family;
 		((struct sockaddr_in *)&rpc->s)->sin_port   = htons(port);
-		((struct sockaddr_in *)&rpc->s)->sin_addr   = ((struct sockaddr_in *)(ai->ai_addr))->sin_addr;
+		((struct sockaddr_in *)&rpc->s)->sin_addr   = ((struct sockaddr_in *)(void *)(ai->ai_addr))->sin_addr;
 #ifdef HAVE_SOCKADDR_LEN
 		((struct sockaddr_in *)&rpc->s)->sin_len = sizeof(struct sockaddr_in);
 #endif
@@ -585,7 +583,7 @@ int rpc_connect_async(struct rpc_context *rpc, const char *server, int port, rpc
 	case AF_INET6:
 		((struct sockaddr_in6 *)&rpc->s)->sin6_family = ai->ai_family;
 		((struct sockaddr_in6 *)&rpc->s)->sin6_port   = htons(port);
-		((struct sockaddr_in6 *)&rpc->s)->sin6_addr   = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
+		((struct sockaddr_in6 *)&rpc->s)->sin6_addr   = ((struct sockaddr_in6 *)(void *)(ai->ai_addr))->sin6_addr;
 #ifdef HAVE_SOCKADDR_LEN
 		((struct sockaddr_in6 *)&rpc->s)->sin6_len = sizeof(struct sockaddr_in6);
 #endif
@@ -608,6 +606,10 @@ int rpc_disconnect(struct rpc_context *rpc, const char *error)
 {
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
+	/* Do not re-disconnect if we are already disconnected */
+	if (!rpc->is_connected) {
+		return 0;
+	}
 	rpc_unset_autoreconnect(rpc);
 
 	if (rpc->fd != -1) {
